package cn.skcks.docking.gb28181.wvp.service.video;

import jakarta.servlet.AsyncContext;
import jakarta.servlet.ServletResponse;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.bytedeco.ffmpeg.avcodec.AVPacket;
import org.bytedeco.ffmpeg.global.avcodec;
import org.bytedeco.ffmpeg.global.avutil;
import org.bytedeco.javacv.FFmpegFrameGrabber;
import org.bytedeco.javacv.FFmpegFrameRecorder;
import org.bytedeco.javacv.FrameGrabber;
import org.springframework.stereotype.Service;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicBoolean;

@Slf4j
@Service
public class RecordService {
    public void header(HttpServletResponse response) {
        response.setContentType("video/x-flv");
        response.setHeader("Accept-Ranges", "none");
        response.setHeader("Connection", "close");
        response.setHeader("Content-Disposition","attachment; filename=\"record.flv\"");
    }

    @SneakyThrows
    public void record(HttpServletRequest request, HttpServletResponse response, String url, long time) {
        AsyncContext asyncContext = request.startAsync();
        asyncContext.setTimeout(0);
        asyncContext.start(() -> {
            header(response);
            try{
                record(asyncContext.getResponse(), url, time);
            } finally {
                log.info("record 结束");
                asyncContext.complete();
            }
        });
    }

    @SneakyThrows
    private void record(ServletResponse response, String url, long time) {
        ByteArrayOutputStream stream = new ByteArrayOutputStream();
        OutputStream outputStream = response.getOutputStream();
        log.info("url {}", url);
        // FFmpeg 调试日志
//        FFmpegLogCallback.set();
        FFmpegFrameGrabber grabber = new FFmpegFrameGrabber(url);
        grabber.start();

        FFmpegFrameRecorder recorder = new FFmpegFrameRecorder(stream, grabber.getImageWidth(), grabber.getImageHeight(), grabber.getAudioChannels());
        recorder.setInterleaved(true);
        recorder.setVideoOption("preset", "ultrafast");
        recorder.setVideoOption("tune", "zerolatency");
        recorder.setVideoOption("crf", "25");
        recorder.setFrameRate(grabber.getFrameRate());
        recorder.setSampleRate(grabber.getSampleRate());
        recorder.setOption("flvflags", "no_duration_filesize");
        if (grabber.getAudioChannels() > 0) {
            recorder.setAudioChannels(grabber.getAudioChannels());
            recorder.setAudioBitrate(grabber.getAudioBitrate());
            recorder.setAudioCodec(grabber.getAudioCodec());
        }
        recorder.setVideoBitrate(grabber.getVideoBitrate());
//        recorder.setVideoCodec(grabber.getVideoCodec());
        recorder.setVideoCodec(avcodec.AV_CODEC_ID_H264);
        recorder.setPixelFormat(avutil.AV_PIX_FMT_YUV420P); // 视频源数据yuv
        recorder.setAudioCodec(avcodec.AV_CODEC_ID_AAC); // 设置音频压缩方式
        recorder.setFormat("flv");
        recorder.setVideoOption("threads", String.valueOf(Runtime.getRuntime().availableProcessors())); // 解码线程数
        recorder.start(grabber.getFormatContext());
        log.info("开始录像");

        ScheduledExecutorService scheduledExecutorService = Executors.newSingleThreadScheduledExecutor();
        AtomicBoolean record = new AtomicBoolean(true);
        scheduledExecutorService.schedule(() -> {
            log.info("到达结束时间, 结束录制");
            record.set(false);
        }, time, TimeUnit.SECONDS);
        try {
            AVPacket k;
            while (record.get() && (k = grabber.grabPacket()) != null) {
                recorder.recordPacket(k);
                if(stream.size() > 0){
                    outputStream.write(stream.toByteArray());
                    outputStream.flush();
                    stream.reset();
                }
                avcodec.av_packet_unref(k);
            }
            grabber.close();
            recorder.close();
        } catch (FFmpegFrameRecorder.Exception | FrameGrabber.Exception e) {
            throw new RuntimeException(e);
        } catch (IOException ignore){}

        log.info("结束录制");
        outputStream.close();
    }
}
